@@ -44,13 +44,23 @@ if 'MKL_NUM_THREADS' not in os.environ:
 
 if 'OPENBLAS_NUM_THREADS' not in os.environ:
     os.environ['OPENBLAS_NUM_THREADS'] = '1'
+
 import sys
-import platform
+import warnings
+
 import qutip.settings
 import qutip.version
 from qutip.version import version as __version__
 from qutip.utilities import _version2int
 
+# -----------------------------------------------------------------------------
+# Check if we're in IPython.
+try:
+    __IPYTHON__
+    qutip.settings.ipython = True
+except:
+    qutip.settings.ipython = False
+
 # -----------------------------------------------------------------------------
 # Check for minimum requirements of dependencies, give the user a warning
 # if the requirements aren't fulfilled
@@ -64,7 +74,7 @@ try:
               ("(%s), requiring %s." %
                (numpy.__version__, numpy_requirement)))
 except:
-    print("QuTiP warning: numpy not found.")
+    warnings.warn("numpy not found.")
 
 scipy_requirement = "0.11.0"
 try:
@@ -74,7 +84,7 @@ try:
               ("(%s), requiring %s." %
                (scipy.__version__, scipy_requirement)))
 except:
-    print("QuTiP warning: scipy not found.")
+    warnings.warn("scipy not found.")
 
 # -----------------------------------------------------------------------------
 # check to see if running from install directory for released versions.
@@ -90,6 +100,7 @@ else:
               "Change directories before running QuTiP.")
     setup_file.close()
 
+del top_path
 
 # -----------------------------------------------------------------------------
 # setup the cython environment
@@ -108,33 +119,36 @@ try:
 
 except Exception as e:
     print("QuTiP warning: Cython setup failed: " + str(e))
-
-
-# -----------------------------------------------------------------------------
-# default configuration settings
-#
-
-import multiprocessing
-
-# load cpus
-from qutip.hardware_info import hardware_info
-info = hardware_info()
-if 'cpus' in info:
-    qutip.settings.num_cpus = info['cpus']
 else:
-    qutip.settings.num_cpus = multiprocessing.cpu_count()
-
-qutip.settings.qutip_graphics = "YES"
+    del Cython, pyximport
 
 # -----------------------------------------------------------------------------
 # Load user configuration if present: override defaults.
 #
 try:
-    qutip_rc_file = os.environ['HOME'] + "/.qutiprc"
+    if os.name == "nt":
+        qutip_rc_file = os.path.join(
+            os.getenv('APPDATA'), 'qutip', "qutiprc"
+        )
+    else:
+        qutip_rc_file = os.path.join(
+            # This should possibly be changed to ~/.config/qutiprc,
+            # to follow XDG specs. Also, OS X uses a different naming
+            # convention as well.
+            os.environ['HOME'], ".qutiprc"
+        )
     qutip.settings.load_rc_file(qutip_rc_file)
 
 except Exception as e:
-    pass
+    try:
+        qutip.settings._logger.warning("Error loading RC file.", exc_info=1)
+    except:
+        pass
+
+# -----------------------------------------------------------------------------
+# cpu/process configuration
+#
+import multiprocessing
 
 # Check if environ flag for qutip processes is set
 if 'QUTIP_NUM_PROCESSES' in os.environ:
@@ -142,30 +156,28 @@ if 'QUTIP_NUM_PROCESSES' in os.environ:
 else:
     os.environ['QUTIP_NUM_PROCESSES'] = str(qutip.settings.num_cpus)
 
+if qutip.settings.num_cpus == 0:
+    # if num_cpu is 0 set it to the available number of cores
+    from qutip.hardware_info import hardware_info
+    info = hardware_info()
+    if 'cpus' in info:
+        qutip.settings.num_cpus = info['cpus']
+    else:
+        qutip.settings.num_cpus = multiprocessing.cpu_count()
+
+
 # -----------------------------------------------------------------------------
 # Load configuration from environment variables: override defaults and
 # configuration file.
 #
 
-if 'QUTIP_GRAPHICS' not in os.environ:
-    os.environ['QUTIP_GRAPHICS'] = qutip.settings.qutip_graphics
-else:
-    qutip.settings.qutip_graphics = os.environ['QUTIP_GRAPHICS']
-
-# check if being run remotely
-if sys.platform not in ['darwin', 'win32'] and 'DISPLAY' not in os.environ:
-    # no graphics if DISPLAY isn't set
-    os.environ['QUTIP_GRAPHICS'] = "NO"
-    qutip.settings.qutip_graphics = "NO"
-
 # check for fortran mcsolver files
 try:
-    from qutip.fortran import qutraj_run
+    from qutip.fortran import mcsolve_f90
 except:
     qutip.settings.fortran = False
 else:
     qutip.settings.fortran = True
-    from qutip.fortran import *
 
 # check for scikits.umfpack
 try:
@@ -174,16 +186,24 @@ except:
     qutip.settings.umfpack = False
 else:
     qutip.settings.umfpack = True
+    del umfpack
 # -----------------------------------------------------------------------------
 # Check that import modules are compatible with requested configuration
 #
 
+
 # Check for Matplotlib
 try:
     import matplotlib
 except:
-    os.environ['QUTIP_GRAPHICS'] = "NO"
-    qutip.settings.qutip_graphics = 'NO'
+    warnings.warn("matplotlib not found: Graphics will not work.")
+else:
+    del matplotlib
+
+# -----------------------------------------------------------------------------
+# Clean name space
+#
+del os, sys, numpy, scipy, multiprocessing
 
 # -----------------------------------------------------------------------------
 # Load modules
@@ -193,68 +213,54 @@ except:
 from qutip.qobj import *
 from qutip.states import *
 from qutip.operators import *
-from qutip.superoperator import *
 from qutip.expect import *
 from qutip.tensor import *
-from qutip.parfor import *
+from qutip.superoperator import *
+from qutip.superop_reps import *
+from qutip.subsystem_apply import *
+from qutip.graph import *
 
 # graphics
-if qutip.settings.qutip_graphics == 'YES':
-    from qutip.bloch import Bloch
-    from qutip.visualization import (
-        hinton, wigner_cmap, sphereplot,
-        energy_level_diagram, plot_energy_levels,
-        fock_distribution, plot_fock_distribution,
-        wigner_fock_distribution, plot_wigner_fock_distribution, plot_wigner,
-        plot_expectation_values, plot_spin_distribution_2d,
-        plot_spin_distribution_3d,
-        plot_qubism, plot_schmidt, complex_array_to_rgb)
-
-    from qutip.orbital import *
-    # load mayavi dependent functions if available
-    try:
-        import mayavi
-    except:
-        pass
-    else:
-        from qutip.bloch3d import Bloch3d
+from qutip.bloch import *
+from qutip.visualization import *
+from qutip.orbital import *
+from qutip.bloch3d import *
+from qutip.matplotlib_utilities import *
 
 # library functions
 from qutip.tomography import *
 from qutip.wigner import *
 from qutip.random_objects import *
 from qutip.simdiag import *
-from qutip.entropy import (entropy_vn, entropy_linear, entropy_mutual,
-                           concurrence, entropy_conditional, entangling_power)
-from qutip.metrics import (fidelity, tracedist, average_gate_fidelity,
-                           process_fidelity)
-from qutip.partial_transpose import partial_transpose
-from qutip.permute import reshuffle
+from qutip.entropy import *
+from qutip.metrics import *
+from qutip.partial_transpose import *
+from qutip.permute import *
 from qutip.continuous_variables import *
 from qutip.distributions import *
+from qutip.three_level_atom import *
 
 # evolution
-from qutip.solver import Options, Odeoptions, Odedata
-from qutip.rhs_generate import rhs_generate, rhs_clear
-from qutip.mesolve import mesolve, odesolve
-from qutip.sesolve import sesolve
-from qutip.mcsolve import mcsolve
-from qutip.stochastic import ssesolve, ssepdpsolve, smesolve, smepdpsolve
+from qutip.solver import *
+from qutip.rhs_generate import *
+from qutip.mesolve import *
+from qutip.sesolve import *
+from qutip.mcsolve import *
+from qutip.stochastic import *
 from qutip.essolve import *
 from qutip.eseries import *
-from qutip.steadystate import *
-from qutip.correlation import *
 from qutip.propagator import *
 from qutip.floquet import *
 from qutip.bloch_redfield import *
-from qutip.superop_reps import *
-from qutip.subsystem_apply import subsystem_apply
-from qutip.graph import *
+from qutip.steadystate import *
+from qutip.correlation import *
+from qutip.countstat import *
 
 # quantum information
 from qutip.qip import *
 
 # utilities
+from qutip.parallel import *
 from qutip.utilities import *
 from qutip.fileio import *
 from qutip.about import *
