@@ -35,21 +35,23 @@ This module provides solvers for the Lindblad master equation and von Neumann
 equation.
 """
 
+__all__ = ['mesolve', 'odesolve']
+
 import os
 import types
 from functools import partial
 import numpy as np
 import scipy.sparse as sp
 import scipy.integrate
-from scipy.linalg import norm
 import warnings
 
 from qutip.qobj import Qobj, isket, isoper, issuper
 from qutip.superoperator import spre, spost, liouvillian, mat2vec, vec2mat
-from qutip.expect import expect, expect_rho_vec
+from qutip.expect import expect_rho_vec
 from qutip.solver import Options, Result, config
 from qutip.cy.spmatfuncs import cy_ode_rhs, cy_ode_rho_func_td
 from qutip.cy.codegen import Codegen
+from qutip.cy.utilities import _cython_build_cleanup
 from qutip.rhs_generate import rhs_generate
 from qutip.states import ket2dm
 from qutip.rhs_generate import _td_format_check, _td_wrap_array_str
@@ -58,7 +60,7 @@ from qutip.settings import debug
 from qutip.sesolve import (_sesolve_list_func_td, _sesolve_list_str_td,
                            _sesolve_list_td, _sesolve_func_td, _sesolve_const)
 
-from qutip.ui.progressbar import BaseProgressBar
+from qutip.ui.progressbar import BaseProgressBar, TextProgressBar
 
 if debug:
     import inspect
@@ -69,13 +71,14 @@ if debug:
 # any collapse operators were given.
 #
 def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
-            progress_bar=BaseProgressBar()):
+            progress_bar=None):
     """
-    Master equation evolution of a density matrix for a given Hamiltonian.
+    Master equation evolution of a density matrix for a given Hamiltonian and
+    set of collapse operators, or a Liouvillian.
 
     Evolve the state vector or density matrix (`rho0`) using a given
     Hamiltonian (`H`) and an [optional] set of collapse operators
-    (`c_op_list`), by integrating the set of ordinary differential equations
+    (`c_ops`), by integrating the set of ordinary differential equations
     that define the system. In the absence of collapse operators the system is
     evolved according to the unitary evolution of the Hamiltonian.
 
@@ -85,9 +88,15 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
     time in `tlist` with time and the state as arguments, and the function
     does not use any return values.
 
+    If either `H` or the Qobj elements in `c_ops` are superoperators, they
+    will be treated as direct contributions to the total system Liouvillian.
+    This allows to solve master equations that are not on standard Lindblad
+    form by passing a custom Liouvillian in place of either the `H` or `c_ops`
+    elements.
+
     **Time-dependent operators**
 
-    For problems with time-dependent problems `H` and `c_ops` can be callback
+    For time-dependent problems, `H` and `c_ops` can be callback
     functions that takes two arguments, time and `args`, and returns the
     Hamiltonian or Liouvillian for the system at that point in time
     (*callback format*).
@@ -97,7 +106,7 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
     operator (:class:`qutip.qobj`) at the first element and where the
     second element is either a string (*list string format*), a callback
     function (*list callback format*) that evaluates to the time-dependent
-    coefficient for the corresponding operator, or a numpy array (*list
+    coefficient for the corresponding operator, or a NumPy array (*list
     array format*) which specifies the value of the coefficient to the
     corresponding operator for each value of t in tlist.
 
@@ -105,24 +114,32 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
 
         H = [[H0, 'sin(w*t)'], [H1, 'sin(2*w*t)']]
 
-        H = [[H0, sin(w*tlist)], [H1, sin(2*w*tlist)]]
-
         H = [[H0, f0_t], [H1, f1_t]]
 
         where f0_t and f1_t are python functions with signature f_t(t, args).
 
+        H = [[H0, np.sin(w*tlist)], [H1, np.sin(2*w*tlist)]]
+
     In the *list string format* and *list callback format*, the string
     expression and the callback function must evaluate to a real or complex
     number (coefficient for the corresponding operator).
 
     In all cases of time-dependent operators, `args` is a dictionary of
     parameters that is used when evaluating operators. It is passed to the
-    callback functions as second argument
+    callback functions as second argument.
+
+    **Additional options**
+
+    Additional options to mesolve can be set via the `options` argument, which
+    should be an instance of :class:`qutip.solver.Options`. Many ODE
+    integration options can be set this way, and the `store_states` and
+    `store_final_state` options can be used to store states even though
+    expectation values are requested via the `e_ops` argument.
 
     .. note::
 
         If an element in the list-specification of the Hamiltonian or
-        the list of collapse operators are in super-operator for it will be
+        the list of collapse operators are in superoperator form it will be
         added to the total Liouvillian of the problem with out further
         transformation. This allows for using mesolve for solving master
         equations that are not on standard Lindblad form.
@@ -143,8 +160,8 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
     ----------
 
     H : :class:`qutip.Qobj`
-        system Hamiltonian, or a callback function for time-dependent
-        Hamiltonians.
+        System Hamiltonian, or a callback function for time-dependent
+        Hamiltonians, or alternatively a system Liouvillian.
 
     rho0 : :class:`qutip.Qobj`
         initial density matrix or state vector (ket).
@@ -153,7 +170,8 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
         list of times for :math:`t`.
 
     c_ops : list of :class:`qutip.Qobj`
-        single collapse operator, or list of collapse operators.
+        single collapse operator, or list of collapse operators, or a list
+        of Liouvillian superoperators.
 
     e_ops : list of :class:`qutip.Qobj` / callback function single
         single operator or list of operators for which to evaluate
@@ -164,22 +182,31 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
         collapse operators.
 
     options : :class:`qutip.Options`
-        with options for the ODE solver.
+        with options for the solver.
+
+    progress_bar: BaseProgressBar
+        Optional instance of BaseProgressBar, or a subclass thereof, for
+        showing the progress of the simulation.
 
     Returns
     -------
 
-    output: :class:`qutip.solver`
+    result: :class:`qutip.Result`
 
-        An instance of the class :class:`qutip.solver`, which contains either
-        an *array* of expectation values for the times specified by `tlist`, or
-        an *array* or state vectors or density matrices corresponding to the
-        times in `tlist` [if `e_ops` is an empty list], or
-        nothing if a callback function was given in place of operators for
-        which to calculate the expectation values.
+        An instance of the class :class:`qutip.Result`, which contains
+        either an *array* `result.expect` of expectation values for the times
+        specified by `tlist`, or an *array* `result.states` of state vectors or
+        density matrices corresponding to the times in `tlist` [if `e_ops` is
+        an empty list], or nothing if a callback function was given in place of
+        operators for which to calculate the expectation values.
 
     """
 
+    if progress_bar is None:
+        progress_bar = BaseProgressBar()
+    elif progress_bar is True:
+        progress_bar = TextProgressBar()
+
     # check whether c_ops or e_ops is is a single operator
     # if so convert it to a list containing only that operator
     if isinstance(c_ops, Qobj):
@@ -198,7 +225,7 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
     H, c_ops, args = _td_wrap_array_str(H, c_ops, args, tlist)
 
     # check for type (if any) of time-dependent inputs
-    n_const, n_func, n_str = _td_format_check(H, c_ops)
+    _, n_func, n_str = _td_format_check(H, c_ops)
 
     if options is None:
         options = Options()
@@ -250,7 +277,8 @@ def mesolve(H, rho0, tlist, c_ops, e_ops, args={}, options=None,
 
         elif isinstance(H, (types.FunctionType,
                             types.BuiltinFunctionType, partial)):
-            # old style time-dependence: must have constant collapse operators
+            # function-callback style time-dependence: must have constant
+            # collapse operators
             if n_str > 0:  # or n_func > 0:
                 raise TypeError("Incorrect format: function-format " +
                                 "Hamiltonian cannot be mixed with " +
@@ -372,7 +400,6 @@ def _mesolve_list_func_td(H_list, rho0, tlist, c_list, e_ops, args, opt,
                             "collapse operators (expected callback function)")
 
         if isoper(c):
-            cdc = c.dag() * c
             L_list.append([liouvillian(None, [c], data_only=True),
                            c_coeff, c_square])
 
@@ -633,10 +660,11 @@ def _mesolve_const(H, rho0, tlist, c_op_list, e_ops, args, opt,
         # if initial state is a ket and no collapse operator where given,
         # fall back on the unitary schrodinger equation solver
         if len(c_op_list) == 0 and isoper(H):
-            return _sesolve_const(H, rho0, tlist, e_ops, args, opt)
+            return _sesolve_const(H, rho0, tlist, e_ops, args, opt,
+                                  progress_bar)
 
         # Got a wave function as initial state: convert to density matrix.
-        rho0 = rho0 * rho0.dag()
+        rho0 = ket2dm(rho0)
 
     #
     # construct liouvillian
@@ -835,7 +863,9 @@ def _generic_ode_solve(r, rho0, tlist, e_ops, opt, progress_bar):
         progress_bar.update(t_idx)
 
         if not r.successful():
-            break
+            raise Exception("ODE integration error: Try to increase "
+                            "the allowed number of substeps by increasing "
+                            "the nsteps parameter in the Options class.")
 
         if opt.store_states or expt_callback:
             rho.data = vec2mat(r.y)
@@ -861,10 +891,7 @@ def _generic_ode_solve(r, rho0, tlist, e_ops, opt, progress_bar):
     progress_bar.finished()
 
     if not opt.rhs_reuse and config.tdname is not None:
-        try:
-            os.remove(config.tdname + ".pyx")
-        except:
-            pass
+        _cython_build_cleanup(config.tdname)
 
     if opt.store_final_state:
         rho.data = vec2mat(r.y)
@@ -898,7 +925,8 @@ def _mesolve_list_td(H_func, rho0, tlist, c_op_list, e_ops, args, opt,
         # if initial state is a ket and no collapse operator where given,
         # fall back on the unitary schrodinger equation solver
         if len(c_op_list) == 0:
-            return _sesolve_list_td(H_func, rho0, tlist, e_ops, args, opt)
+            return _sesolve_list_td(H_func, rho0, tlist, e_ops, args, opt,
+                                    progress_bar)
 
         # Got a wave function as initial state: convert to density matrix.
         rho0 = ket2dm(rho0)
@@ -923,9 +951,9 @@ def _mesolve_list_td(H_func, rho0, tlist, c_op_list, e_ops, args, opt,
     lenh = len(H_func[0])
     if opt.tidy:
         H_func[0] = [(H_func[0][k]).tidyup() for k in range(lenh)]
-    L_func = [[liouvillian_fast(H_func[0][0], c_op_list)], H_func[1]]
+    L_func = [[liouvillian(H_func[0][0], c_op_list)], H_func[1]]
     for m in range(1, lenh):
-        L_func[0].append(liouvillian_fast(H_func[0][m], []))
+        L_func[0].append(liouvillian(H_func[0][m], []))
 
     # create data arrays for time-dependent RHS function
     Ldata = [L_func[0][k].data.data for k in range(lenh)]
