@@ -35,8 +35,9 @@ This module contains a collection of functions for calculating metrics
 (distance measures) between states and operators.
 """
 
-from qutip.qobj import *
-import scipy.linalg as la
+__all__ = ['fidelity', 'tracedist', 'bures_dist', 'bures_angle',
+           'hilbert_dist', 'average_gate_fidelity', 'process_fidelity']
+
 import numpy as np
 from qutip.sparse import sp_eigs
 from qutip.states import ket2dm
@@ -62,8 +63,8 @@ def fidelity(A, B):
 
     Examples
     --------
-    >>> x=fock_dm(5,3)
-    >>> y=coherent_dm(5,1)
+    >>> x = fock_dm(5,3)
+    >>> y = coherent_dm(5,1)
     >>> fidelity(x,y)
     0.24104350624628332
 
@@ -122,7 +123,7 @@ def tracedist(A, B, sparse=False, tol=0):
     See: Nielsen & Chuang, "Quantum Computation and Quantum Information"
 
     Parameters
-    ----------
+    ----------!=
     A : qobj
         Density matrix or state vector.
     B : qobj
@@ -175,6 +176,10 @@ def hilbert_dist(A, B):
     dist : float
         Hilbert-Schmidt distance between density matrices.
 
+    Notes
+    -----
+    See V. Vedral and M. B. Plenio, Phys. Rev. A 57, 1619 (1998).
+
     """
     if A.isket or A.isbra:
         A = ket2dm(A)
@@ -184,7 +189,7 @@ def hilbert_dist(A, B):
     if A.dims != B.dims:
         raise TypeError('A and B do not have same dimensions.')
 
-    return (A - B).norm('fro')
+    return ((A - B)**2).tr()
 
 
 def bures_dist(A, B):
