@@ -30,6 +30,17 @@
 #    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
+"""
+This module contains functions for generating Qobj representation of a variety
+of commonly occuring quantum operators.
+"""
+
+__all__ = ['jmat', 'spin_Jx', 'spin_Jy', 'spin_Jz', 'spin_Jm', 'spin_Jp',
+           'spin_J_set', 'sigmap', 'sigmam', 'sigmax', 'sigmay', 'sigmaz',
+           'destroy', 'create', 'qeye', 'identity', 'position', 'momentum',
+           'num', 'squeeze', 'squeezing', 'displace', 'commutator',
+           'qutrit_ops', 'qdiags', 'phase', 'zero_oper', 'enr_destroy',
+           'enr_identity']
 
 import numpy as np
 import scipy
@@ -113,6 +124,10 @@ shape = [3, 3], type = oper, isHerm = True
 
 
 def _jplus(j):
+    """
+    Internal functions for generating the data representing the J-plus
+    operator.
+    """
     m = np.arange(j, -j - 1, -1)
     N = len(m)
     return sp.spdiags(np.sqrt(j * (j + 1.0) - (m + 1.0) * m),
@@ -120,6 +135,9 @@ def _jplus(j):
 
 
 def _jz(j):
+    """
+    Internal functions for generating the data representing the J-z operator.
+    """
     m = np.arange(j, -j - 1, -1)
     N = len(m)
     return sp.spdiags(m, 0, N, N, format='csr')
@@ -401,8 +419,10 @@ def qeye(N):
 
     Parameters
     ----------
-    N : int
-        Dimension of Hilbert space.
+    N : int or list of ints
+        Dimension of Hilbert space. If provided as a list of ints,
+        then the dimension is the product over this list, but the
+        ``dims`` property of the new Qobj are set to this list.
 
     Returns
     -------
@@ -420,6 +440,8 @@ shape = [3, 3], type = oper, isHerm = True
      [ 0.  0.  1.]]
 
     """
+    if isinstance(N, list):
+        return tensor.tensor(*[identity(n) for n in N])
     N = int(N)
     if (not isinstance(N, (int, np.integer))) or N < 0:
         raise ValueError("N must be integer N>=0")
@@ -431,8 +453,10 @@ def identity(N):
 
     Parameters
     ----------
-    N : int
-        Dimension of Hilbert space.
+    N : int or list of ints
+        Dimension of Hilbert space. If provided as a list of ints,
+        then the dimension is the product over this list, but the
+        ``dims`` property of the new Qobj are set to this list.
 
     Returns
     -------
@@ -444,7 +468,7 @@ def identity(N):
 
 def position(N, offset=0):
     """
-    Position operator x=1\sqrt(2)*(a+a.dag())
+    Position operator x=1/sqrt(2)*(a+a.dag())
 
     Parameters
     ----------
@@ -519,7 +543,7 @@ shape = [4, 4], type = oper, isHerm = True
     return Qobj(data)
 
 
-def squeeze(N, sp, offset=0):
+def squeeze(N, z, offset=0):
     """Single-mode Squeezing operator.
 
 
@@ -528,7 +552,7 @@ def squeeze(N, sp, offset=0):
     N : int
         Dimension of hilbert space.
 
-    sp : float/complex
+    z : float/complex
         Squeezing parameter.
 
     offset : int (default 0)
@@ -543,7 +567,7 @@ def squeeze(N, sp, offset=0):
 
     Examples
     --------
-    >>> squeeze(4,0.25)
+    >>> squeeze(4, 0.25)
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isHerm = False
     Qobj data =
@@ -554,7 +578,7 @@ shape = [4, 4], type = oper, isHerm = False
 
     """
     a = destroy(N, offset=offset)
-    op = (1 / 2.0) * np.conj(sp) * (a ** 2) - (1 / 2.0) * sp * (a.dag()) ** 2
+    op = (1 / 2.0) * np.conj(z) * (a ** 2) - (1 / 2.0) * z * (a.dag()) ** 2
     return op.expm()
 
 
@@ -564,7 +588,7 @@ def squeezing(a1, a2, z):
     .. math::
 
         S(z) = \\exp\\left(\\frac{1}{2}\\left(z^*a_1a_2
-        - za_1^\dagger a_2^\dagger\\right)\\right)
+        - za_1^\\dagger a_2^\\dagger\\right)\\right)
 
     Parameters
     ----------
@@ -658,7 +682,8 @@ def qutrit_ops():
     sig12 = one * two.dag()
     sig23 = two * three.dag()
     sig31 = three * one.dag()
-    return np.array([sig11, sig22, sig33, sig12, sig23, sig31])
+    return np.array([sig11, sig22, sig33, sig12, sig23, sig31],
+                    dtype=object)
 
 
 def qdiags(diagonals, offsets, dims=None, shape=None):
@@ -703,7 +728,7 @@ shape = [4, 4], type = oper, isherm = False
     try:
         data = sp.diags(diagonals, offsets, shape, format='csr', dtype=complex)
     except:
-        raise NotImplemented("This function required SciPy 0.11+.")
+        raise NotImplementedError("This function requires SciPy 0.11+.")
     if not dims:
         dims = [[], []]
     if not shape:
@@ -759,3 +784,100 @@ def zero_oper(N, dims=None):
 
     """
     return Qobj(sp.csr_matrix((N, N), dtype=complex), dims=dims)
+
+
+def enr_destroy(dims, excitations):
+    """
+    Generate annilation operators for modes in a excitation-number-restricted
+    state space. For example, consider a system consisting of 4 modes, each
+    with 5 states. The total hilbert space size is 5**4 = 625. If we are
+    only interested in states that contain up to 2 excitations, we only need
+    to include states such as
+
+        (0, 0, 0, 0)
+        (0, 0, 0, 1)
+        (0, 0, 0, 2)
+        (0, 0, 1, 0)
+        (0, 0, 1, 1)
+        (0, 0, 2, 0)
+        ...
+
+    This function creates annihilation operators for the 4 modes that act
+    within this state space:
+
+        a1, a2, a3, a4 = enr_destroy([5, 5, 5, 5], excitations=2)
+
+    From this point onwards, the annihiltion operators a1, ..., a4 can be
+    used to setup a Hamiltonian, collapse operators and expectation-value
+    operators, etc., following the usual pattern.
+
+    Parameters
+    ----------
+    dims : list
+        A list of the dimensions of each subsystem of a composite quantum
+        system.
+
+    excitations : integer
+        The maximum number of excitations that are to be included in the
+        state space.
+
+    Returns
+    -------
+    a_ops : list of qobj
+        A list of annihilation operators for each mode in the composite
+        quantum system described by dims.
+    """
+    from qutip.states import enr_state_dictionaries
+
+    nstates, state2idx, idx2state = enr_state_dictionaries(dims, excitations)
+
+    a_ops = [sp.lil_matrix((nstates, nstates), dtype=np.complex)
+             for _ in range(len(dims))]
+
+    for n1, state1 in idx2state.items():
+        for n2, state2 in idx2state.items():
+            for idx, a in enumerate(a_ops):
+                s1 = [s for idx2, s in enumerate(state1) if idx != idx2]
+                s2 = [s for idx2, s in enumerate(state2) if idx != idx2]
+                if (state1[idx] == state2[idx] - 1) and (s1 == s2):
+                    a_ops[idx][n1, n2] = np.sqrt(state2[idx])
+
+    return [Qobj(a, dims=[dims, dims]) for a in a_ops]
+
+
+def enr_identity(dims, excitations):
+    """
+    Generate the identity operator for the excitation-number restricted
+    state space defined by the `dims` and `exciations` arguments. See the
+    docstring for enr_fock for a more detailed description of these arguments.
+
+    Parameters
+    ----------
+    dims : list
+        A list of the dimensions of each subsystem of a composite quantum
+        system.
+
+    excitations : integer
+        The maximum number of excitations that are to be included in the
+        state space.
+
+    state : list of integers
+        The state in the number basis representation.
+
+    Returns
+    -------
+    op : Qobj
+        A Qobj instance that represent the identity operator in the
+        exication-number-restricted state space defined by `dims` and
+        `exciations`.
+    """
+    from qutip.states import enr_state_dictionaries
+
+    nstates, _, _ = enr_state_dictionaries(dims, excitations)
+    data = sp.eye(nstates, nstates, dtype=np.complex)
+    return Qobj(data, dims=[dims, dims])
+
+# Break circular dependencies by a trailing import.
+# Note that we use a relative import here to deal with that
+# qutip.tensor is the *function* tensor, not the module.
+from . import tensor
