@@ -30,12 +30,16 @@
 #    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
-"""Main module for QuTiP, consisting of the Quantum Object (Qobj) class and
-its methods.
+"""The Quantum Object (Qobj) class, for representing quantum states and
+operators, and related functions.
 """
+
+__all__ = ['Qobj', 'qobj_list_evaluate', 'ptrace', 'dag', 'isequal',
+           'issuper', 'isoper', 'isoperket', 'isoperbra', 'isket', 'isbra',
+           'isherm', 'shape', 'dims']
+
 import warnings
 import types
-import pickle
 
 try:
     import builtins
@@ -56,7 +60,7 @@ from qutip import __version__
 from qutip.ptrace import _ptrace
 from qutip.permute import _permute
 from qutip.sparse import (sp_eigs, sp_expm, sp_fro_norm, sp_max_norm,
-                          sp_one_norm, sp_L2_norm, sp_inf_norm)
+                          sp_one_norm, sp_L2_norm)
 
 
 class Qobj(object):
@@ -220,16 +224,15 @@ class Qobj(object):
 
         elif isinstance(inpt, list) or isinstance(inpt, tuple):
             # case where input is a list
-            if len(np.array(inpt).shape) == 1:
+            data = np.array(inpt)
+            if len(data.shape) == 1:
                 # if list has only one dimension (i.e [5,4])
-                inpt = np.array([inpt]).transpose()
-            else:  # if list has two dimensions (i.e [[5,4]])
-                inpt = np.array(inpt)
+                data = data.transpose()
 
-            self.data = sp.csr_matrix(inpt, dtype=complex)
+            self.data = sp.csr_matrix(data, dtype=complex)
 
             if not np.any(dims):
-                self.dims = [[int(inpt.shape[0])], [int(inpt.shape[1])]]
+                self.dims = [[int(data.shape[0])], [int(data.shape[1])]]
             else:
                 self.dims = dims
 
@@ -433,7 +436,8 @@ class Qobj(object):
 
         elif isinstance(other, (list, np.ndarray)):
             # if other is a list, do element-wise multiplication
-            return np.array([self * item for item in other])
+            return np.array([self * item for item in other],
+                            dtype=object)
 
         elif isinstance(other, eseries):
             return other.__rmul__(self)
@@ -461,7 +465,8 @@ class Qobj(object):
 
         if isinstance(other, (list, np.ndarray)):
             # if other is a list, do element-wise multiplication
-            return np.array([item * self for item in other])
+            return np.array([item * self for item in other],
+                            dtype=object)
 
         if isinstance(other, eseries):
             return other.__mul__(self)
@@ -1048,7 +1053,7 @@ class Qobj(object):
         -------
         P : qobj
             Permuted quantum object.
-i
+
         """
         q = Qobj()
         q.data, q.dims = _permute(self, order)
@@ -1274,7 +1279,8 @@ i
                                sort=sort, eigvals=eigvals, tol=tol,
                                maxiter=maxiter)
         new_dims = [self.dims[0], [1] * len(self.dims[0])]
-        ekets = np.array([Qobj(vec, dims=new_dims) for vec in evecs])
+        ekets = np.array([Qobj(vec, dims=new_dims) for vec in evecs],
+                         dtype=object)
         norms = np.array([ket.norm() for ket in ekets])
         return evals, ekets / norms
 
@@ -1512,7 +1518,6 @@ i
 
     @property
     def type(self):
-
         if not self._type:
             if self.isoper:
                 self._type = 'oper'
@@ -1533,7 +1538,10 @@ i
 
     @property
     def shape(self):
-        return [np.prod(self.dims[0]), np.prod(self.dims[1])]
+        if self.data.shape == (1, 1):
+            return [np.prod(self.dims[0]), np.prod(self.dims[1])]
+        else:
+            return list(self.data.shape)
 
     @property
     def isbra(self):
