@@ -34,16 +34,16 @@
 This module provides solvers for the unitary Schrodinger equation.
 """
 
+__all__ = ['sesolve']
+
 import os
 import types
 from functools import partial
 import numpy as np
-import scipy.sparse as sp
 import scipy.integrate
 from scipy.linalg import norm
 
 from qutip.qobj import Qobj, isket
-from qutip.expect import expect
 from qutip.rhs_generate import rhs_generate
 from qutip.solver import Result, Options, config
 from qutip.rhs_generate import _td_format_check, _td_wrap_array_str
@@ -635,7 +635,9 @@ def _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar,
         progress_bar.update(t_idx)
 
         if not r.successful():
-            break
+            raise Exception("ODE integration error: Try to increase "
+                            "the allowed number of substeps by increasing "
+                            "the nsteps parameter in the Options class.")
 
         if state_norm_func:
             data = r.y / state_norm_func(r.y)
