@@ -30,8 +30,10 @@
 #    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
-
 from __future__ import print_function
+
+__all__ = ['Options', 'Odeoptions', 'Odedata']
+
 import os
 import warnings
 from qutip import __version__
@@ -83,7 +85,10 @@ class Options():
     average_states : bool {False}
         Average states values over trajectories in stochastic solvers.
     average_expect : bool {True}
-        Avgerage expectation values over trajectories for stochastic solvers.
+        Average expectation values over trajectories for stochastic solvers.
+    mc_corr_eps : float {1e-10}
+        Arbitrarily small value for eliminating any divide-by-zero errors in
+        correlation calculations when using mcsolve.
     ntraj : int {500}
         Number of trajectories in stochastic solvers.
     rhs_reuse : bool {False,True}
@@ -93,6 +98,8 @@ class Options():
         callback signature.
     rhs_filename : str
         Name for compiled Cython file.
+    seeds : ndarray
+        Array containing random number seeds for mcsolver.
     store_final_state : bool {False, True}
         Whether or not to store the final state of the evolution in the
         result class.
@@ -101,7 +108,9 @@ class Options():
         result class, even if expectation values operators are given. If no
         expectation are provided, then states are stored by default and this
         option has no effect.
+
     """
+
     def __init__(self, atol=1e-8, rtol=1e-6, method='adams', order=12,
                  nsteps=1000, first_step=0, max_step=0, min_step=0,
                  average_expect=True, average_states=False, tidy=True,
@@ -131,6 +140,8 @@ class Options():
         self.average_expect = average_expect
         # Number of trajectories (default = 500)
         self.ntraj = ntraj
+        # Holds seeds for rand num gen
+        self.seeds = seeds
         # tidyup Hamiltonian before calculation (default = True)
         self.tidy = tidy
         # include the state in the function callback signature
@@ -140,13 +151,11 @@ class Options():
         # Use filename for preexisting RHS function (will default to last
         # compiled function if None & rhs_exists=True)
         self.rhs_filename = rhs_filename
+        # small value in mc solver for computing correlations
+        self.mc_corr_eps = 1e-10
         # Number of processors to use (mcsolve only)
         if num_cpus:
             self.num_cpus = num_cpus
-            if self.num_cpus > int(os.environ['QUTIP_NUM_PROCESSES']):
-                message = ("Requested number of threads larger than number " +
-                           "of CPUs (%s)." % os.environ['QUTIP_NUM_PROCESSES'])
-                warnings.warn(message)
         else:
             self.num_cpus = 0
         # Tolerance for wavefunction norm (mcsolve only)
@@ -158,12 +167,14 @@ class Options():
         self.store_final_state = store_final_state
         # store states even if expectation operators are given?
         self.store_states = store_states
-        # extra solver parameters
-        self.seeds = seeds
         # average mcsolver density matricies assuming steady state evolution
         self.steady_state_average = steady_state_average
 
     def __str__(self):
+        if self.seeds is None:
+            seed_length = 0
+        else:
+            seed_length = len(self.seeds)
         s = ""
         s += "Options:\n"
         s += "-----------\n"
@@ -181,6 +192,7 @@ class Options():
         s += "norm_steps:        " + str(self.norm_steps) + "\n"
         s += "rhs_filename:      " + str(self.rhs_filename) + "\n"
         s += "rhs_reuse:         " + str(self.rhs_reuse) + "\n"
+        s += "seeds:             " + str(seed_length) + "\n"
         s += "rhs_with_state:    " + str(self.rhs_with_state) + "\n"
         s += "average_expect:    " + str(self.average_expect) + "\n"
         s += "average_states:    " + str(self.average_states) + "\n"
@@ -228,6 +240,7 @@ class Result():
         self.num_expect = 0
         self.num_collapse = 0
         self.ntraj = None
+        self.seeds = None
         self.col_times = None
         self.col_which = None
 
@@ -281,7 +294,6 @@ class SolverConfiguration():
         self.options = None     # options for solvers
         self.norm_tol = None    # tolerance for wavefunction norm
         self.norm_steps = None  # max. number of steps to take in finding
-
         # Initial state stuff
         self.psi0 = None        # initial state
         self.psi0_dims = None   # initial state dims
@@ -344,6 +356,7 @@ class SolverConfiguration():
         self.c_funcs = None
         self.c_func_args = None
 
+
 #
 # create a global instance of the SolverConfiguration class
 #
