@@ -35,16 +35,23 @@ This module contains a collection of routines for operating on sparse
 matrices on the scipy.sparse formats, for use internally by other modules
 throughout QuTiP.
 """
+
+__all__ = ['sp_fro_norm', 'sp_inf_norm', 'sp_L2_norm', 'sp_max_norm',
+           'sp_one_norm', 'sp_reshape', 'sp_eigs', 'sp_expm', 'sp_permute',
+           'sp_reverse_permute', 'sp_bandwidth', 'sp_profile']
+
 import scipy.sparse as sp
 import scipy.sparse.linalg as spla
 import numpy as np
 import scipy.linalg as la
 from scipy.linalg.blas import get_blas_funcs
 _dznrm2 = get_blas_funcs("znrm2")
-from qutip.cy.sparse_utils import (
-    _sparse_permute, _sparse_reverse_permute, _sparse_bandwidth)
+from qutip.cy.sparse_utils import (_sparse_profile, _sparse_permute,
+                                   _sparse_reverse_permute, _sparse_bandwidth)
 from qutip.settings import debug
 
+import qutip.logging
+logger = qutip.logging.get_logger()
 
 if debug:
     import inspect
@@ -151,7 +158,7 @@ def _dense_eigs(data, isherm, vecs, N, eigvals, num_large, num_small):
     matrix.
     """
     if debug:
-        print(inspect.stack()[0][3] + ": vectors = " + str(vecs))
+        logger.debug(inspect.stack()[0][3] + ": vectors = " + str(vecs))
 
     evecs = None
 
@@ -608,3 +615,30 @@ def sp_bandwidth(A):
         return mb, lb, ub
     else:
         raise Exception('Invalid sparse input format.')
+
+
+def sp_profile(A):
+    """Returns the total, lower, and upper profiles of a sparse matrix.
+
+    If the matrix is symmetric then the upper and lower profiles are
+    identical. Diagonal matrices have zero profile.
+
+    Parameters
+    ----------
+    A : csr_matrix, csc_matrix
+        Input matrix
+    """
+    if sp.isspmatrix_csr(A):
+        up = _sparse_profile(A.indices, A.indptr, A.shape[0])
+        A = A.tocsc()
+        lp = _sparse_profile(A.indices, A.indptr, A.shape[0])
+
+    elif sp.isspmatrix_csc(A):
+        lp = _sparse_profile(A.indices, A.indptr, A.shape[0])
+        A = A.tocsr()
+        up = _sparse_profile(A.indices, A.indptr, A.shape[0])
+
+    else:
+        raise TypeError('Input sparse matrix must be in CSR or CSC format.')
+
+    return up+lp, lp, up
