@@ -30,35 +30,37 @@
 #    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
-import numpy as np
-import scipy.sparse as sp
-import unittest
-from numpy.testing import assert_, run_module_suite, assert_equal
-from numpy.testing.decorators import skipif
-from scipy.sparse.csgraph import breadth_first_order as BFO
 import os
+import numpy as np
+from numpy.testing import run_module_suite, assert_equal
+import unittest
+import scipy.sparse as sp
+from scipy.sparse.csgraph import breadth_first_order as BFO
+
 pwd = os.path.dirname(__file__)
-from qutip import *
+
+from qutip import (rand_dm, graph_degree, breadth_first_search,
+                   reverse_cuthill_mckee, tensor, destroy, qeye, sigmam,
+                   liouvillian, maximum_bipartite_matching,
+                   weighted_bipartite_matching)
 from qutip.sparse import sp_permute, sp_bandwidth
 
-# find networkx if it exists
-try:
-    import networkx as nx
-except:
-    nx_found = 0
-else:
-    nx_found = 1
 
-
-@unittest.skipIf(nx_found == 0, 'Networkx not installed.')
 def test_graph_degree():
     "Graph: Graph Degree"
-    A = rand_dm(25, 0.5)
+    A = rand_dm(25, 0.1)
     deg = graph_degree(A.data)
-    G = nx.from_scipy_sparse_matrix(A.data)
-    nx_deg = G.degree()
-    nx_deg = array([nx_deg[k] for k in range(25)])
-    assert_equal((deg - nx_deg).all(), 0)
+    A = A.full()
+    np_deg = []
+    for k in range(25):
+        num = 0
+        inds = A[k, :].nonzero()[0]
+        num = len(inds)
+        if k in inds:
+            num += 1
+        np_deg.append(num)
+    np.asarray(np_deg, dtype=int)
+    assert_equal((deg - np_deg).all(), 0)
 
 
 def test_graph_bfs():
@@ -74,17 +76,17 @@ def test_graph_bfs():
 
 def test_graph_rcm_simple():
     "Graph: Reverse Cuthill-McKee Ordering (simple)"
-    A = array([[1, 0, 0, 0, 1, 0, 0, 0],
-               [0, 1, 1, 0, 0, 1, 0, 1],
-               [0, 1, 1, 0, 1, 0, 0, 0],
-               [0, 0, 0, 1, 0, 0, 1, 0],
-               [1, 0, 1, 0, 1, 0, 0, 0],
-               [0, 1, 0, 0, 0, 1, 0, 1],
-               [0, 0, 0, 1, 0, 0, 1, 0],
-               [0, 1, 0, 0, 0, 1, 0, 1]], dtype=np.int32)
+    A = np.array([[1, 0, 0, 0, 1, 0, 0, 0],
+                  [0, 1, 1, 0, 0, 1, 0, 1],
+                  [0, 1, 1, 0, 1, 0, 0, 0],
+                  [0, 0, 0, 1, 0, 0, 1, 0],
+                  [1, 0, 1, 0, 1, 0, 0, 0],
+                  [0, 1, 0, 0, 0, 1, 0, 1],
+                  [0, 0, 0, 1, 0, 0, 1, 0],
+                  [0, 1, 0, 0, 0, 1, 0, 1]], dtype=np.int32)
     A = sp.csr_matrix(A)
     perm = reverse_cuthill_mckee(A)
-    ans = array([6, 3, 7, 5, 1, 2, 4, 0])
+    ans = np.array([6, 3, 7, 5, 1, 2, 4, 0])
     assert_equal((perm - ans).all(), 0)
 
 
@@ -110,12 +112,13 @@ def test_graph_rcm_boost():
     M = M+M.T
     M = sp.csr_matrix(M)
     perm = reverse_cuthill_mckee(M, 1)
-    ans_perm = array([9, 7, 6, 4, 1, 5, 0, 2, 3, 8])
+    ans_perm = np.array([9, 7, 6, 4, 1, 5, 0, 2, 3, 8])
     assert_equal((perm - ans_perm).all(), 0)
     P = sp_permute(M, perm, perm)
     bw = sp_bandwidth(P)
     assert_equal(bw[2], 4)
 
+
 def test_graph_rcm_qutip():
     "Graph: Reverse Cuthill-McKee Ordering (qutip)"
     kappa = 1
@@ -126,11 +129,12 @@ def test_graph_rcm_qutip():
     E = 1.5
     a = tensor(destroy(N), qeye(2))
     sm = tensor(qeye(N), sigmam())
-    H = (w0-wl)*sm.dag()*sm+(wc-wl)*a.dag()*a+1j*g*(a.dag()*sm-sm.dag()*a)+E*(a.dag()+a)
-    c_ops = [sqrt(2*kappa)*a, sqrt(gamma)*sm]
-    L = liouvillian(H,c_ops)
+    H = (w0-wl)*sm.dag(
+        )*sm+(wc-wl)*a.dag()*a+1j*g*(a.dag()*sm-sm.dag()*a)+E*(a.dag()+a)
+    c_ops = [np.sqrt(2*kappa)*a, np.sqrt(gamma)*sm]
+    L = liouvillian(H, c_ops)
     perm = reverse_cuthill_mckee(L.data)
-    ans = np.array([12,14,4,6,10,8,2,15,0,13,7,5,9,11,1,3])
+    ans = np.array([12, 14, 4, 6, 10, 8, 2, 15, 0, 13, 7, 5, 9, 11, 1, 3])
     assert_equal(perm, ans)
 
 
