@@ -31,12 +31,14 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
-from qutip import *
-from qutip import _version2int
-from numpy import allclose, linspace, mean, ones
+import numpy as np
 from numpy.testing import assert_equal, run_module_suite, assert_
-from numpy.testing.decorators import skipif
 import unittest
+
+from qutip import (mcsolve, destroy, basis, qeye, Options, tensor, sigmam,
+                   expect, coherent, Qobj)
+from qutip import _version2int
+
 # find Cython if it exists
 try:
     import Cython
@@ -49,11 +51,11 @@ kappa = 0.2
 
 
 def sqrt_kappa(t, args):
-    return sqrt(kappa)
+    return np.sqrt(kappa)
 
 
 def sqrt_kappa2(t, args):
-    return sqrt(kappa * exp(-t))
+    return np.sqrt(kappa * np.exp(-t))
 
 
 def const_H1_coeff(t, args):
@@ -71,13 +73,12 @@ def test_MCNoCollExpt():
     a = destroy(N)
     H = a.dag() * a
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -88,14 +89,35 @@ def test_MCNoCollStates():
     a = destroy(N)
     H = a.dag() * a
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [], ntraj=ntraj)
     states = mcdata.states
     expt = expect(a.dag() * a, states)
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
+    assert_equal(diff < error, True)
+
+
+def test_MCNoCollExpectStates():
+    "Monte-carlo: Constant H with no collapse ops (expect and states)"
+    error = 1e-8
+    N = 10  # number of basis states to consider
+    a = destroy(N)
+    H = a.dag() * a
+    psi0 = basis(N, 9)  # initial state
+    c_op_list = []
+    tlist = np.linspace(0, 10, 100)
+    mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj,
+                     options=Options(store_states=True))
+    actual_answer = 9.0 * np.ones(len(tlist))
+    expt = mcdata.expect[0]
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
+    assert_equal(diff < error, True)
+    assert_(len(mcdata.states) == len(tlist))
+    assert_(isinstance(mcdata.states[0], Qobj))
+    expt = expect(a.dag() * a, mcdata.states)
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -106,14 +128,13 @@ def test_MCNoCollStrExpt():
     a = destroy(N)
     H = [a.dag() * a, [a.dag() * a, 'c']]
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], args={'c': 0.0},
                      ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -124,13 +145,12 @@ def test_MCNoCollFuncExpt():
     a = destroy(N)
     H = [a.dag() * a, [a.dag() * a, const_H1_coeff]]
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -141,14 +161,13 @@ def test_MCNoCollStrStates():
     a = destroy(N)
     H = [a.dag() * a, [a.dag() * a, 'c']]
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [], args={'c': 0.0})
     states = mcdata.states
     expt = expect(a.dag() * a, states)
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -159,14 +178,13 @@ def test_MCNoCollFuncStates():
     a = destroy(N)
     H = [a.dag() * a, [a.dag() * a, const_H1_coeff]]
     psi0 = basis(N, 9)  # initial state
-    kappa = 0.2  # coupling to oscillator
     c_op_list = []
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [], ntraj=ntraj)
     states = mcdata.states
     expt = expect(a.dag() * a, states)
-    actual_answer = 9.0 * ones(len(tlist))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.ones(len(tlist))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -174,11 +192,11 @@ def test_MCCollapseTimesOperators():
     "Monte-carlo: Check for stored collapse operators and times"
     N = 10
     kappa = 5.0
-    times = linspace(0, 10, 100)
+    times = np.linspace(0, 10, 100)
     a = destroy(N)
     H = a.dag() * a
     psi0 = basis(N, 9)
-    c_ops = [sqrt(kappa) * a, sqrt(kappa) * a]
+    c_ops = [np.sqrt(kappa) * a, np.sqrt(kappa) * a]
     result = mcsolve(H, psi0, times, c_ops, [], ntraj=1)
     assert_(len(result.col_times[0]) > 0)
     assert_(len(result.col_which) == len(result.col_times))
@@ -192,12 +210,53 @@ def test_MCSimpleConst():
     H = a.dag() * a
     psi0 = basis(N, 9)  # initial state
     kappa = 0.2  # coupling to oscillator
-    c_op_list = [sqrt(kappa) * a]
-    tlist = linspace(0, 10, 100)
+    c_op_list = [np.sqrt(kappa) * a]
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * tlist)
-    avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
+    assert_equal(avg_diff < mc_error, True)
+
+
+def test_MCSimpleConstStates():
+    "Monte-carlo: Constant H with constant collapse (states)"
+    N = 10  # number of basis states to consider
+    a = destroy(N)
+    H = a.dag() * a
+    psi0 = basis(N, 9)  # initial state
+    kappa = 0.2  # coupling to oscillator
+    c_op_list = [np.sqrt(kappa) * a]
+    tlist = np.linspace(0, 10, 100)
+    mcdata = mcsolve(H, psi0, tlist, c_op_list, [], ntraj=ntraj,
+                     options=Options(average_states=True))
+    assert_(len(mcdata.states) == len(tlist))
+    assert_(isinstance(mcdata.states[0], Qobj))
+    expt = expect(a.dag() * a, mcdata.states)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
+    assert_equal(avg_diff < mc_error, True)
+
+
+def test_MCSimpleConstExptStates():
+    "Monte-carlo: Constant H with constant collapse (states)"
+    N = 10  # number of basis states to consider
+    a = destroy(N)
+    H = a.dag() * a
+    psi0 = basis(N, 9)  # initial state
+    kappa = 0.2  # coupling to oscillator
+    c_op_list = [np.sqrt(kappa) * a]
+    tlist = np.linspace(0, 10, 100)
+    mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj,
+                     options=Options(average_states=True, store_states=True))
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    expt1 = mcdata.expect[0]
+    avg_diff = np.mean(abs(actual_answer - expt1) / actual_answer)
+    assert_equal(avg_diff < mc_error, True)
+    assert_(len(mcdata.states) == len(tlist))
+    assert_(isinstance(mcdata.states[0], Qobj))
+    expt2 = expect(a.dag() * a, mcdata.states)
+    avg_diff = np.mean(abs(actual_answer - expt2) / actual_answer)
     assert_equal(avg_diff < mc_error, True)
 
 
@@ -208,12 +267,12 @@ def test_MCSimpleSingleCollapse():
     H = a.dag() * a
     psi0 = basis(N, 9)  # initial state
     kappa = 0.2  # coupling to oscillator
-    c_op_list = sqrt(kappa) * a
-    tlist = linspace(0, 10, 100)
+    c_op_list = np.sqrt(kappa) * a
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * tlist)
-    avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(avg_diff < mc_error, True)
 
 
@@ -224,12 +283,12 @@ def test_MCSimpleSingleExpect():
     H = a.dag() * a
     psi0 = basis(N, 9)  # initial state
     kappa = 0.2  # coupling to oscillator
-    c_op_list = [sqrt(kappa) * a]
-    tlist = linspace(0, 10, 100)
+    c_op_list = [np.sqrt(kappa) * a]
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, a.dag() * a, ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * tlist)
-    avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(avg_diff < mc_error, True)
 
 
@@ -241,11 +300,11 @@ def test_MCSimpleConstFunc():
     psi0 = basis(N, 9)  # initial state
     kappa = 0.2  # coupling to oscillator
     c_op_list = [[a, sqrt_kappa]]
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * tlist)
-    avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(avg_diff < mc_error, True)
 
 
@@ -260,12 +319,12 @@ def test_MCSimpleConstStr():
     kappa = 0.2  # coupling to oscillator
     c_op_list = [[a, 'sqrt(k)']]
     args = {'k': kappa}
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], args=args,
                      ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * tlist)
-    avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * tlist)
+    avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(avg_diff < mc_error, True)
 
 
@@ -278,11 +337,11 @@ def test_MCTDFunc():
     psi0 = basis(N, 9)  # initial state
     kappa = 0.2  # coupling to oscillator
     c_op_list = [[a, sqrt_kappa2]]
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual_answer = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-    diff = mean(abs(actual_answer - expt) / actual_answer)
+    actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+    diff = np.mean(abs(actual_answer - expt) / actual_answer)
     assert_equal(diff < error, True)
 
 
@@ -298,12 +357,12 @@ def test_TDStr():
     kappa = 0.2  # coupling to oscillator
     c_op_list = [[a, 'sqrt(k*exp(-t))']]
     args = {'k': kappa}
-    tlist = linspace(0, 10, 100)
+    tlist = np.linspace(0, 10, 100)
     mcdata = mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], args=args,
                      ntraj=ntraj)
     expt = mcdata.expect[0]
-    actual = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-    diff = mean(abs(actual - expt) / actual)
+    actual = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+    diff = np.mean(abs(actual - expt) / actual)
     assert_equal(diff < error, True)
 
 
@@ -318,7 +377,7 @@ def test_mc_dtypes1():
     wl = 0  # driving frequency
     E = 0.5  # driving amplitude
     N = 5  # number of cavity energy levels (0->3 Fock states)
-    tlist = linspace(0, 10, 5)  # times for expectation values
+    tlist = np.linspace(0, 10, 5)  # times for expectation values
     # construct Hamiltonian
     ida = qeye(N)
     idatom = qeye(2)
@@ -327,8 +386,8 @@ def test_mc_dtypes1():
     H = (w0 - wl) * sm.dag() * sm + (wc - wl) * a.dag() * a + \
         1j * g * (a.dag() * sm - sm.dag() * a) + E * (a.dag() + a)
     # collapse operators
-    C1 = sqrt(2 * kappa) * a
-    C2 = sqrt(gamma) * sm
+    C1 = np.sqrt(2 * kappa) * a
+    C2 = np.sqrt(gamma) * sm
     C1dC1 = C1.dag() * C1
     C2dC2 = C2.dag() * C2
     # intial state
@@ -352,7 +411,7 @@ def test_mc_dtypes2():
     wl = 0  # driving frequency
     E = 0.5  # driving amplitude
     N = 5  # number of cavity energy levels (0->3 Fock states)
-    tlist = linspace(0, 10, 5)  # times for expectation values
+    tlist = np.linspace(0, 10, 5)  # times for expectation values
     # construct Hamiltonian
     ida = qeye(N)
     idatom = qeye(2)
@@ -361,8 +420,8 @@ def test_mc_dtypes2():
     H = (w0 - wl) * sm.dag() * sm + (wc - wl) * a.dag() * a + \
         1j * g * (a.dag() * sm - sm.dag() * a) + E * (a.dag() + a)
     # collapse operators
-    C1 = sqrt(2 * kappa) * a
-    C2 = sqrt(gamma) * sm
+    C1 = np.sqrt(2 * kappa) * a
+    C2 = np.sqrt(gamma) * sm
     C1dC1 = C1.dag() * C1
     C2dC2 = C2.dag() * C2
     # intial state
@@ -374,5 +433,107 @@ def test_mc_dtypes2():
     assert_equal(isinstance(data.expect[0][1][1], float), True)
     assert_equal(isinstance(data.expect[0][2][1], complex), True)
 
+
+def test_mc_seed_reuse():
+    "Monte-carlo: check reusing seeds"
+    N0 = 6
+    N1 = 6
+    N2 = 6
+    # damping rates
+    gamma0 = 0.1
+    gamma1 = 0.4
+    gamma2 = 0.1
+    alpha = np.sqrt(2)  # initial coherent state param for mode 0
+    tlist = np.linspace(0, 10, 2)
+    ntraj = 500  # number of trajectories
+    # define operators
+    a0 = tensor(destroy(N0), qeye(N1), qeye(N2))
+    a1 = tensor(qeye(N0), destroy(N1), qeye(N2))
+    a2 = tensor(qeye(N0), qeye(N1), destroy(N2))
+    # number operators for each mode
+    num0 = a0.dag() * a0
+    num1 = a1.dag() * a1
+    num2 = a2.dag() * a2
+    # dissipative operators for zero-temp. baths
+    C0 = np.sqrt(2.0 * gamma0) * a0
+    C1 = np.sqrt(2.0 * gamma1) * a1
+    C2 = np.sqrt(2.0 * gamma2) * a2
+    # initial state: coherent mode 0 & vacuum for modes #1 & #2
+    psi0 = tensor(coherent(N0, alpha), basis(N1, 0), basis(N2, 0))
+    # trilinear Hamiltonian
+    H = 1j * (a0 * a1.dag() * a2.dag() - a0.dag() * a1 * a2)
+    # run Monte-Carlo
+    data1 = mcsolve(H, psi0, tlist, [C0, C1, C2], [num0, num1, num2],
+                    ntraj=ntraj)
+    data2 = mcsolve(H, psi0, tlist, [C0, C1, C2], [num0, num1, num2],
+                    ntraj=ntraj, options=Options(seeds=data1.seeds))
+    for k in range(ntraj):
+        assert_equal(np.allclose(data1.col_times[k],data2.col_times[k]), True)
+        assert_equal(np.allclose(data1.col_which[k],data2.col_which[k]), True)
+
+
+def test_mc_seed_noreuse():
+    "Monte-carlo: check not reusing seeds"
+    N0 = 6
+    N1 = 6
+    N2 = 6
+    # damping rates
+    gamma0 = 0.1
+    gamma1 = 0.4
+    gamma2 = 0.1
+    alpha = np.sqrt(2)  # initial coherent state param for mode 0
+    tlist = np.linspace(0, 10, 2)
+    ntraj = 500  # number of trajectories
+    # define operators
+    a0 = tensor(destroy(N0), qeye(N1), qeye(N2))
+    a1 = tensor(qeye(N0), destroy(N1), qeye(N2))
+    a2 = tensor(qeye(N0), qeye(N1), destroy(N2))
+    # number operators for each mode
+    num0 = a0.dag() * a0
+    num1 = a1.dag() * a1
+    num2 = a2.dag() * a2
+    # dissipative operators for zero-temp. baths
+    C0 = np.sqrt(2.0 * gamma0) * a0
+    C1 = np.sqrt(2.0 * gamma1) * a1
+    C2 = np.sqrt(2.0 * gamma2) * a2
+    # initial state: coherent mode 0 & vacuum for modes #1 & #2
+    psi0 = tensor(coherent(N0, alpha), basis(N1, 0), basis(N2, 0))
+    # trilinear Hamiltonian
+    H = 1j * (a0 * a1.dag() * a2.dag() - a0.dag() * a1 * a2)
+    # run Monte-Carlo
+    data1 = mcsolve(H, psi0, tlist, [C0, C1, C2], [num0, num1, num2],
+                    ntraj=ntraj)
+    data2 = mcsolve(H, psi0, tlist, [C0, C1, C2], [num0, num1, num2],
+                    ntraj=ntraj)
+    diff_flag = False
+    for k in range(ntraj):
+        if len(data1.col_times[k]) != len(data2.col_times[k]):
+            diff_flag = 1
+            break
+        else:
+            if not np.allclose(data1.col_which[k],data2.col_which[k]):
+                diff_flag = 1
+                break
+    assert_equal(diff_flag, 1)
+
+
+def test_mc_ntraj_list():
+    "Monte-carlo: list of trajectories"
+    N = 5
+    a = destroy(N)
+    H = a.dag()*a       # Simple oscillator Hamiltonian
+    psi0 = basis(N, 1)  # Initial Fock state with one photon
+    kappa = 1.0/0.129   # Coupling rate to heat bath
+    nth = 0.063         # Temperature with <n>=0.063
+    # Build collapse operators for the thermal bath
+    c_ops = []
+    c_ops.append(np.sqrt(kappa * (1 + nth)) * a)
+    c_ops.append(np.sqrt(kappa * nth) * a.dag())
+    ntraj = [1, 5, 15, 904]  # number of MC trajectories
+    tlist = np.linspace(0, 0.8, 100)
+    mc = mcsolve(H, psi0, tlist, c_ops, [a.dag()*a], ntraj)
+    assert_equal(len(mc.expect), 4)
+
+
 if __name__ == "__main__":
     run_module_suite()
