@@ -32,14 +32,16 @@
 ###############################################################################
 
 from functools import partial
-from numpy import allclose, linspace, mean, ones
+
+import numpy as np
 from numpy.testing import assert_, run_module_suite
 
 # disable the MC progress bar
 import os
 os.environ['QUTIP_GRAPHICS'] = "NO"
 
-from qutip import *
+from qutip import (sigmax, sigmay, sigmaz, sigmam, mesolve, tensor, destroy,
+                   identity, steadystate, expect, basis, num)
 
 
 class TestJCModelEvolution:
@@ -55,11 +57,11 @@ class TestJCModelEvolution:
 
         rate = g1
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sigmam())
+            c_op_list.append(np.sqrt(rate) * sigmam())
 
         rate = g2
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sigmaz())
+            c_op_list.append(np.sqrt(rate) * sigmaz())
 
         output = mesolve(
             H, psi0, tlist, c_op_list, [sigmax(), sigmay(), sigmaz()])
@@ -71,8 +73,8 @@ class TestJCModelEvolution:
                        pump, psi0, use_rwa, tlist):
 
         # Hamiltonian
-        a = tensor(destroy(N), qeye(2))
-        sm = tensor(qeye(N), destroy(2))
+        a = tensor(destroy(N), identity(2))
+        sm = tensor(identity(N), destroy(2))
 
         if use_rwa:
             # use the rotating wave approxiation
@@ -88,19 +90,19 @@ class TestJCModelEvolution:
         n_th_a = 0.0  # zero temperature
 
         rate = kappa * (1 + n_th_a)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
 
         rate = kappa * n_th_a
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * a.dag())
+            c_op_list.append(np.sqrt(rate) * a.dag())
 
         rate = gamma
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sm)
+            c_op_list.append(np.sqrt(rate) * sm)
 
         rate = pump
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sm.dag())
+            c_op_list.append(np.sqrt(rate) * sm.dag())
 
         # find the steady state
         rho_ss = steadystate(H, c_op_list)
@@ -111,13 +113,13 @@ class TestJCModelEvolution:
                      pump, psi0, use_rwa, tlist):
 
         # Hamiltonian
-        a = tensor(destroy(N), qeye(2))
-        sm = tensor(qeye(N), destroy(2))
+        a = tensor(destroy(N), identity(2))
+        sm = tensor(identity(N), destroy(2))
 
         if use_rwa:
             # use the rotating wave approxiation
-            H = wc * a.dag(
-            ) * a + wa * sm.dag() * sm + g * (a.dag() * sm + a * sm.dag())
+            H = wc * a.dag() * a + wa * sm.dag() * sm + g * (
+                a.dag() * sm + a * sm.dag())
         else:
             H = wc * a.dag() * a + wa * sm.dag() * sm + g * (
                 a.dag() + a) * (sm + sm.dag())
@@ -128,19 +130,19 @@ class TestJCModelEvolution:
         n_th_a = 0.0  # zero temperature
 
         rate = kappa * (1 + n_th_a)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
 
         rate = kappa * n_th_a
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * a.dag())
+            c_op_list.append(np.sqrt(rate) * a.dag())
 
         rate = gamma
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sm)
+            c_op_list.append(np.sqrt(rate) * sm)
 
         rate = pump
         if rate > 0.0:
-            c_op_list.append(sqrt(rate) * sm.dag())
+            c_op_list.append(np.sqrt(rate) * sm.dag())
 
         # evolve and calculate expectation values
         output = mesolve(
@@ -151,38 +153,38 @@ class TestJCModelEvolution:
     def testQubitDynamics1(self):
         "mesolve: qubit with dissipation"
 
-        epsilon = 0.0 * 2 * pi   # cavity frequency
-        delta = 1.0 * 2 * pi   # atom frequency
+        epsilon = 0.0 * 2 * np.pi   # cavity frequency
+        delta = 1.0 * 2 * np.pi   # atom frequency
         g2 = 0.1
         g1 = 0.0
         psi0 = basis(2, 0)        # initial state
-        tlist = linspace(0, 5, 200)
+        tlist = np.linspace(0, 5, 200)
 
         sx, sy, sz = self.qubit_integrate(tlist, psi0, epsilon, delta, g1, g2)
 
-        sx_analytic = zeros(shape(tlist))
-        sy_analytic = -sin(2 * pi * tlist) * exp(-tlist * g2)
-        sz_analytic = cos(2 * pi * tlist) * exp(-tlist * g2)
+        sx_analytic = np.zeros(np.shape(tlist))
+        sy_analytic = -np.sin(2 * np.pi * tlist) * np.exp(-tlist * g2)
+        sz_analytic = np.cos(2 * np.pi * tlist) * np.exp(-tlist * g2)
 
         assert_(max(abs(sx - sx_analytic)) < 0.05)
         assert_(max(abs(sy - sy_analytic)) < 0.05)
         assert_(max(abs(sz - sz_analytic)) < 0.05)
 
-    def testQubitDynamics1(self):
+    def testQubitDynamics2(self):
         "mesolve: qubit without dissipation"
 
-        epsilon = 0.0 * 2 * pi   # cavity frequency
-        delta = 1.0 * 2 * pi   # atom frequency
+        epsilon = 0.0 * 2 * np.pi   # cavity frequency
+        delta = 1.0 * 2 * np.pi   # atom frequency
         g2 = 0.0
         g1 = 0.0
         psi0 = basis(2, 0)        # initial state
-        tlist = linspace(0, 5, 200)
+        tlist = np.linspace(0, 5, 200)
 
         sx, sy, sz = self.qubit_integrate(tlist, psi0, epsilon, delta, g1, g2)
 
-        sx_analytic = zeros(shape(tlist))
-        sy_analytic = -sin(2 * pi * tlist) * exp(-tlist * g2)
-        sz_analytic = cos(2 * pi * tlist) * exp(-tlist * g2)
+        sx_analytic = np.zeros(np.shape(tlist))
+        sy_analytic = -np.sin(2 * np.pi * tlist) * np.exp(-tlist * g2)
+        sz_analytic = np.cos(2 * np.pi * tlist) * np.exp(-tlist * g2)
 
         assert_(max(abs(sx - sx_analytic)) < 0.05)
         assert_(max(abs(sy - sy_analytic)) < 0.05)
@@ -193,9 +195,9 @@ class TestJCModelEvolution:
 
         use_rwa = True
         N = 4           # number of cavity fock states
-        wc = 2 * pi * 1.0   # cavity frequency
-        wa = 2 * pi * 1.0   # atom frequency
-        g = 2 * pi * 0.01  # coupling strength
+        wc = 2 * np.pi * 1.0   # cavity frequency
+        wa = 2 * np.pi * 1.0   # atom frequency
+        g = 2 * np.pi * 0.01  # coupling strength
         kappa = 0.0     # cavity dissipation rate
         gamma = 0.0     # atom dissipation rate
         pump = 0.0     # atom pump rate
@@ -203,13 +205,13 @@ class TestJCModelEvolution:
         # start with an excited atom and maximum number of photons
         n = N - 2
         psi0 = tensor(basis(N, n), basis(2, 1))
-        tlist = linspace(0, 1000, 2000)
+        tlist = np.linspace(0, 1000, 2000)
 
         nc, na = self.jc_integrate(
             N, wc, wa, g, kappa, gamma, pump, psi0, use_rwa, tlist)
 
-        nc_ex = (n + 0.5 * (1 - cos(2 * g * sqrt(n + 1) * tlist)))
-        na_ex = 0.5 * (1 + cos(2 * g * sqrt(n + 1) * tlist))
+        nc_ex = (n + 0.5 * (1 - np.cos(2 * g * np.sqrt(n + 1) * tlist)))
+        na_ex = 0.5 * (1 + np.cos(2 * g * np.sqrt(n + 1) * tlist))
 
         assert_(max(abs(nc - nc_ex)) < 0.005, True)
         assert_(max(abs(na - na_ex)) < 0.005, True)
@@ -219,9 +221,9 @@ class TestJCModelEvolution:
 
         use_rwa = True
         N = 4           # number of cavity fock states
-        wc = 2 * pi * 1.0   # cavity frequency
-        wa = 2 * pi * 1.0   # atom frequency
-        g = 2 * pi * 0.0   # coupling strength
+        wc = 2 * np.pi * 1.0   # cavity frequency
+        wa = 2 * np.pi * 1.0   # atom frequency
+        g = 2 * np.pi * 0.0   # coupling strength
         kappa = 0.005   # cavity dissipation rate
         gamma = 0.01    # atom dissipation rate
         pump = 0.0     # atom pump rate
@@ -229,15 +231,15 @@ class TestJCModelEvolution:
         # start with an excited atom and maximum number of photons
         n = N - 2
         psi0 = tensor(basis(N, n), basis(2, 1))
-        tlist = linspace(0, 1000, 2000)
+        tlist = np.linspace(0, 1000, 2000)
 
         nc, na = self.jc_integrate(
             N, wc, wa, g, kappa, gamma, pump, psi0, use_rwa, tlist)
 
-        nc_ex = (n + 0.5 * (1 - cos(2 * g * sqrt(n + 1) * tlist))) * \
-            exp(-kappa * tlist)
-        na_ex = 0.5 * (1 + cos(2 * g * sqrt(n + 1) * tlist)) * \
-            exp(-gamma * tlist)
+        nc_ex = (n + 0.5 * (1 - np.cos(2 * g * np.sqrt(n + 1) * tlist))) * \
+            np.exp(-kappa * tlist)
+        na_ex = 0.5 * (1 + np.cos(2 * g * np.sqrt(n + 1) * tlist)) * \
+            np.exp(-gamma * tlist)
 
         assert_(max(abs(nc - nc_ex)) < 0.005, True)
         assert_(max(abs(na - na_ex)) < 0.005, True)
@@ -247,9 +249,9 @@ class TestJCModelEvolution:
 
         use_rwa = True
         N = 4           # number of cavity fock states
-        wc = 2 * pi * 1.0   # cavity frequency
-        wa = 2 * pi * 1.0   # atom frequency
-        g = 2 * pi * 0.1   # coupling strength
+        wc = 2 * np.pi * 1.0   # cavity frequency
+        wa = 2 * np.pi * 1.0   # atom frequency
+        g = 2 * np.pi * 0.1   # coupling strength
         kappa = 0.05    # cavity dissipation rate
         gamma = 0.001   # atom dissipation rate
         pump = 0.25    # atom pump rate
@@ -257,7 +259,7 @@ class TestJCModelEvolution:
         # start with an excited atom and maximum number of photons
         n = N - 2
         psi0 = tensor(basis(N, n), basis(2, 1))
-        tlist = linspace(0, 200, 500)
+        tlist = np.linspace(0, 200, 500)
 
         nc, na = self.jc_integrate(
             N, wc, wa, g, kappa, gamma, pump, psi0, use_rwa, tlist)
@@ -267,8 +269,8 @@ class TestJCModelEvolution:
         nc_ss, na_ss = self.jc_steadystate(
             N, wc, wa, g, kappa, gamma, pump, psi0, use_rwa, tlist)
 
-        nc_ss = nc_ss * ones(shape(nc))
-        na_ss = na_ss * ones(shape(na))
+        nc_ss = nc_ss * np.ones(np.shape(nc))
+        na_ss = na_ss * np.ones(np.shape(na))
 
         assert_(abs(nc[-1] - nc_ss[-1]) < 0.005, True)
         assert_(abs(na[-1] - na_ss[-1]) < 0.005, True)
@@ -282,7 +284,7 @@ class TestMESolverConstDecay:
     A test class for the time-dependent ode check function.
     """
 
-    def testMESimpleConstDecay(self):
+    def testMEDecay(self):
         "mesolve: simple constant decay"
 
         N = 10  # number of basis states to consider
@@ -290,15 +292,15 @@ class TestMESolverConstDecay:
         H = a.dag() * a
         psi0 = basis(N, 9)  # initial state
         kappa = 0.2  # coupling to oscillator
-        c_op_list = [sqrt(kappa) * a]
-        tlist = linspace(0, 10, 100)
+        c_op_list = [np.sqrt(kappa) * a]
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a])
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * tlist)
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * tlist)
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleConstDecaySingleCollapse(self):
+    def testMEDecaySingleCollapse(self):
         "mesolve: simple constant decay"
 
         N = 10  # number of basis states to consider
@@ -306,15 +308,15 @@ class TestMESolverConstDecay:
         H = a.dag() * a
         psi0 = basis(N, 9)  # initial state
         kappa = 0.2  # coupling to oscillator
-        c_op = sqrt(kappa) * a
-        tlist = linspace(0, 10, 100)
+        c_op = np.sqrt(kappa) * a
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op, [a.dag() * a])
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * tlist)
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * tlist)
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleConstDecaySingleExpect(self):
+    def testMEDecaySingleExpect(self):
         "mesolve: simple constant decay"
 
         N = 10  # number of basis states to consider
@@ -322,15 +324,15 @@ class TestMESolverConstDecay:
         H = a.dag() * a
         psi0 = basis(N, 9)  # initial state
         kappa = 0.2  # coupling to oscillator
-        c_op_list = [sqrt(kappa) * a]
-        tlist = linspace(0, 10, 100)
+        c_op_list = [np.sqrt(kappa) * a]
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, a.dag() * a)
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * tlist)
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * tlist)
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleConstDecayAsFuncList(self):
+    def testMEDecayAsFuncList(self):
         "mesolve: constant decay as function list"
 
         N = 10  # number of basis states to consider
@@ -340,16 +342,16 @@ class TestMESolverConstDecay:
         kappa = 0.2  # coupling to oscillator
 
         def sqrt_kappa(t, args):
-            return sqrt(kappa)
+            return np.sqrt(kappa)
         c_op_list = [[a, sqrt_kappa]]
-        tlist = linspace(0, 10, 100)
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a])
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * tlist)
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * tlist)
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleConstDecayAsStrList(self):
+    def testMEDecayAsStrList(self):
         "mesolve: constant decay as string list"
 
         N = 10  # number of basis states to consider
@@ -359,11 +361,11 @@ class TestMESolverConstDecay:
         kappa = 0.2  # coupling to oscillator
         c_op_list = [[a, 'sqrt(k)']]
         args = {'k': kappa}
-        tlist = linspace(0, 10, 100)
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a], args=args)
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * tlist)
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * tlist)
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
 
@@ -379,7 +381,7 @@ class TestMESolveTDDecay:
 
     """
 
-    def testMESimpleTDDecayAsFuncList(self):
+    def testMETDDecayAsFuncList(self):
         "mesolve: time-dependence as function list"
 
         N = 10  # number of basis states to consider
@@ -389,33 +391,34 @@ class TestMESolveTDDecay:
         kappa = 0.2  # coupling to oscillator
 
         def sqrt_kappa(t, args):
-            return sqrt(kappa * exp(-t))
+            return np.sqrt(kappa * np.exp(-t))
         c_op_list = [[a, sqrt_kappa]]
-        tlist = linspace(0, 10, 100)
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a])
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleTDDecayAsPartialFuncList(self):
+    def testMETDDecayAsPartFuncList(self):
         "mesolve: time-dependence as partial function list"
 
         N = 10
         a = destroy(N)
         H = num(N)
         psi0 = basis(N, 9)
-        tlist = linspace(0, 10, 100)
-        c_ops = [[[a, partial(lambda t, args, k: sqrt(k * exp(-t)), k=kappa)]]
+        tlist = np.linspace(0, 10, 100)
+        c_ops = [[[a, partial(lambda t, args, k:
+                              np.sqrt(k * np.exp(-t)), k=kappa)]]
                  for kappa in [0.05, 0.1, 0.2]]
 
         for idx, kappa in enumerate([0.05, 0.1, 0.2]):
             medata = mesolve(H, psi0, tlist, c_ops[idx], [H])
-            ref = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-            avg_diff = mean(abs(ref - medata.expect[0]) / ref)
+            ref = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+            avg_diff = np.mean(abs(ref - medata.expect[0]) / ref)
             assert_(avg_diff < me_error)
 
-    def testMESimpleTDDecayAsStrList(self):
+    def testMETDDecayAsStrList(self):
         "mesolve: time-dependence as string list"
 
         N = 10  # number of basis states to consider
@@ -425,14 +428,14 @@ class TestMESolveTDDecay:
         kappa = 0.2  # coupling to oscillator
         c_op_list = [[a, 'sqrt(k*exp(-t))']]
         args = {'k': kappa}
-        tlist = linspace(0, 10, 100)
+        tlist = np.linspace(0, 10, 100)
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a], args=args)
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < me_error)
 
-    def testMESimpleTDDecayAsArray(self):
+    def testMETDDecayAsArray(self):
         "mesolve: time-dependence as array"
 
         N = 10
@@ -440,12 +443,12 @@ class TestMESolveTDDecay:
         H = a.dag() * a
         psi0 = basis(N, 9)
         kappa = 0.2
-        tlist = linspace(0, 10, 1000)
-        c_op_list = [[a, sqrt(kappa * exp(-tlist))]]
+        tlist = np.linspace(0, 10, 1000)
+        c_op_list = [[a, np.sqrt(kappa * np.exp(-tlist))]]
         medata = mesolve(H, psi0, tlist, c_op_list, [a.dag() * a])
         expt = medata.expect[0]
-        actual_answer = 9.0 * exp(-kappa * (1.0 - exp(-tlist)))
-        avg_diff = mean(abs(actual_answer - expt) / actual_answer)
+        actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
+        avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < 100 * me_error)
 
 if __name__ == "__main__":
