@@ -37,12 +37,13 @@ import numpy as np
 from numpy.testing import assert_equal, assert_, run_module_suite
 
 from qutip.qobj import Qobj
-from qutip.random_objects import rand_ket, rand_dm, rand_herm, rand_unitary
-from qutip.states import basis, fock_dm
+from qutip.random_objects import (rand_ket, rand_dm, rand_herm, rand_unitary,
+                                  rand_super)
+from qutip.states import basis, fock_dm, ket2dm
 from qutip.operators import create, destroy, num, sigmax
 from qutip.superoperator import spre, spost, operator_to_vector
 from qutip.superop_reps import to_super
-from qutip.tensor import tensor, super_tensor
+from qutip.tensor import tensor, super_tensor, composite
 
 from operator import add, mul, truediv, sub
 
@@ -195,10 +196,10 @@ def test_QobjAddition():
     assert_equal(q4_isherm, q4.isherm)
 
     # check elementwise addition/subtraction
-    assert_equal(q3, q4)
+    assert_(q3 == q4)
 
     # check that addition is commutative
-    assert_equal(q1 + q2, q2 + q1)
+    assert_(q1 + q2 == q2 + q1)
 
     data = np.random.random((5, 5))
     q = Qobj(data)
@@ -254,7 +255,7 @@ def test_QobjMultiplication():
 
     q4 = q1 * q2
 
-    assert_equal(q3, q4)
+    assert_(q3 == q4)
 
 
 def test_QobjDivision():
@@ -297,11 +298,11 @@ def test_QobjEquals():
         (5, 5)) + 1j * np.random.random((5, 5)) - (0.5 + 0.5j)
     q1 = Qobj(data)
     q2 = Qobj(data)
-    assert_equal(q1, q2)
+    assert_(q1 == q2)
 
     q1 = Qobj(data)
     q2 = Qobj(-data)
-    assert_equal(q1 != q2, True)
+    assert_(q1 != q2)
 
 
 def test_QobjGetItem():
@@ -432,10 +433,10 @@ def test_QobjEigenStates():
     b, c = A.eigenstates()
     assert_(np.all(b - np.ones(5) == 0))
 
-    kets = np.array([basis(5, k) for k in range(5)])
+    kets = [basis(5, k) for k in range(5)]
 
     for k in range(5):
-        assert_equal(c[k], kets[k])
+        assert_(c[k] == kets[k])
 
 
 def test_QobjExpm():
@@ -511,14 +512,14 @@ def test_QobjPermute():
     C = basis(5, 2)
     psi = tensor(A, B, C)
     psi2 = psi.permute([2, 0, 1])
-    assert_equal(psi2, tensor(C, A, B))
+    assert_(psi2 == tensor(C, A, B))
 
     A = fock_dm(5, 0)
     B = fock_dm(5, 4)
     C = fock_dm(5, 2)
     rho = tensor(A, B, C)
     rho2 = rho.permute([2, 0, 1])
-    assert_equal(rho2, tensor(C, A, B))
+    assert_(rho2 == tensor(C, A, B))
 
     for ii in range(3):
         A = rand_ket(5)
@@ -526,7 +527,7 @@ def test_QobjPermute():
         C = rand_ket(5)
         psi = tensor(A, B, C)
         psi2 = psi.permute([1, 0, 2])
-        assert_equal(psi2, tensor(B, A, C))
+        assert_(psi2 == tensor(B, A, C))
 
     for ii in range(3):
         A = rand_dm(5)
@@ -534,7 +535,7 @@ def test_QobjPermute():
         C = rand_dm(5)
         rho = tensor(A, B, C)
         rho2 = rho.permute([1, 0, 2])
-        assert_equal(rho2, tensor(B, A, C))
+        assert_(rho2 == tensor(B, A, C))
 
 
 def test_KetType():
@@ -642,7 +643,6 @@ def test_arithmetic_preserves_superrep():
                              operation, superrep)
                          )
 
-    dimension = 4
     for superrep in ['super', 'choi', 'chi']:
         for operation, chk_op, chk_scalar in [
                 (add, True, True),
@@ -665,6 +665,15 @@ def test_isherm_skew():
     assert_(tensor(iH, iH).isherm)
 
 
+def test_super_tensor_operket():
+    """
+    Tensor: Checks that super_tensor respects states.
+    """
+    rho1, rho2 = rand_dm(5), rand_dm(7)
+    operator_to_vector(rho1)
+    operator_to_vector(rho2)
+
+
 def test_super_tensor_property():
     """
     Tensor: Super_tensor correctly tensors on underlying spaces.
@@ -677,8 +686,44 @@ def test_super_tensor_property():
 
     S_supertens = super_tensor(to_super(U1), to_super(U2))
 
-    assert_equal(S_tens, S_supertens)
+    assert_(S_tens == S_supertens)
     assert_equal(S_supertens.superrep, 'super')
 
+
+def test_composite_oper():
+    """
+    Composite: Tests compositing unitaries and superoperators.
+    """
+    U1 = rand_unitary(3)
+    U2 = rand_unitary(5)
+    S1 = to_super(U1)
+    S2 = to_super(U2)
+
+    S3 = rand_super(4)
+    S4 = rand_super(7)
+
+    assert_(composite(U1, U2) == tensor(U1, U2))
+    assert_(composite(S3, S4) == super_tensor(S3, S4))
+    assert_(composite(U1, S4) == super_tensor(S1, S4))
+    assert_(composite(S3, U2) == super_tensor(S3, S2))
+
+
+def test_composite_vec():
+    """
+    Composite: Tests compositing states and density operators.
+    """
+    k1 = rand_ket(5)
+    k2 = rand_ket(7)
+    r1 = operator_to_vector(ket2dm(k1))
+    r2 = operator_to_vector(ket2dm(k2))
+
+    r3 = operator_to_vector(rand_dm(3))
+    r4 = operator_to_vector(rand_dm(4))
+
+    assert_(composite(k1, k2) == tensor(k1, k2))
+    assert_(composite(r3, r4) == super_tensor(r3, r4))
+    assert_(composite(k1, r4) == super_tensor(r1, r4))
+    assert_(composite(r3, k2) == super_tensor(r3, r2))
+
 if __name__ == "__main__":
     run_module_suite()
