@@ -32,7 +32,7 @@
 ###############################################################################
 
 from numpy.testing import assert_, run_module_suite
-from qutip import *
+from qutip import coherent_dm, thermal_dm, fock_dm
 
 
 class TestStates:
@@ -73,5 +73,6 @@ class TestStates:
             assert_(abs(rho.tr() - 1.0) < 1e-12)
             assert_(rho.data[i, i] == 1.0)
 
+
 if __name__ == "__main__":
     run_module_suite()
