@@ -31,37 +31,36 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
-
-from qutip import *
-from numpy import linspace
+import numpy as np
 from numpy.testing import assert_, assert_equal, run_module_suite
 
+from qutip import (sigmaz, destroy, steadystate, expect, coherent_dm)
+
 
 def test_qubit_direct():
     "Steady state: Thermal qubit - direct solver"
     # thermal steadystate of a qubit: compare numerics with analytical formula
-    sx = sigmax()
     sz = sigmaz()
     sm = destroy(2)
 
-    H = 0.5 * 2 * pi * sz
+    H = 0.5 * 2 * np.pi * sz
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * sm)
+        c_op_list.append(np.sqrt(rate) * sm)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * sm.dag())
+        c_op_list.append(np.sqrt(rate) * sm.dag())
         rho_ss = steadystate(H, c_op_list, method='direct')
         p_ss[idx] = expect(sm.dag() * sm, rho_ss)
 
-    p_ss_analytic = exp(-1.0 / wth_vec) / (1 + exp(-1.0 / wth_vec))
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-5, True)
 
@@ -69,28 +68,27 @@ def test_qubit_direct():
 def test_qubit_eigen():
     "Steady state: Thermal qubit - eigen solver"
     # thermal steadystate of a qubit: compare numerics with analytical formula
-    sx = sigmax()
     sz = sigmaz()
     sm = destroy(2)
 
-    H = 0.5 * 2 * pi * sz
+    H = 0.5 * 2 * np.pi * sz
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * sm)
+        c_op_list.append(np.sqrt(rate) * sm)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * sm.dag())
+        c_op_list.append(np.sqrt(rate) * sm.dag())
         rho_ss = steadystate(H, c_op_list, method='eigen')
         p_ss[idx] = expect(sm.dag() * sm, rho_ss)
 
-    p_ss_analytic = exp(-1.0 / wth_vec) / (1 + exp(-1.0 / wth_vec))
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-5, True)
 
@@ -98,28 +96,27 @@ def test_qubit_eigen():
 def test_qubit_power():
     "Steady state: Thermal qubit - power solver"
     # thermal steadystate of a qubit: compare numerics with analytical formula
-    sx = sigmax()
     sz = sigmaz()
     sm = destroy(2)
 
-    H = 0.5 * 2 * pi * sz
+    H = 0.5 * 2 * np.pi * sz
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * sm)
+        c_op_list.append(np.sqrt(rate) * sm)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * sm.dag())
+        c_op_list.append(np.sqrt(rate) * sm.dag())
         rho_ss = steadystate(H, c_op_list, method='power')
         p_ss[idx] = expect(sm.dag() * sm, rho_ss)
 
-    p_ss_analytic = exp(-1.0 / wth_vec) / (1 + exp(-1.0 / wth_vec))
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-5, True)
 
@@ -127,28 +124,55 @@ def test_qubit_power():
 def test_qubit_gmres():
     "Steady state: Thermal qubit - iterative-gmres solver"
     # thermal steadystate of a qubit: compare numerics with analytical formula
-    sx = sigmax()
     sz = sigmaz()
     sm = destroy(2)
 
-    H = 0.5 * 2 * pi * sz
+    H = 0.5 * 2 * np.pi * sz
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * sm)
+        c_op_list.append(np.sqrt(rate) * sm)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * sm.dag())
+        c_op_list.append(np.sqrt(rate) * sm.dag())
         rho_ss = steadystate(H, c_op_list, method='iterative-gmres')
         p_ss[idx] = expect(sm.dag() * sm, rho_ss)
 
-    p_ss_analytic = exp(-1.0 / wth_vec) / (1 + exp(-1.0 / wth_vec))
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
+    delta = sum(abs(p_ss_analytic - p_ss))
+    assert_equal(delta < 1e-5, True)
+
+
+def test_qubit_bicgstab():
+    "Steady state: Thermal qubit - iterative-bicgstab solver"
+    # thermal steadystate of a qubit: compare numerics with analytical formula
+    sz = sigmaz()
+    sm = destroy(2)
+
+    H = 0.5 * 2 * np.pi * sz
+    gamma1 = 0.05
+
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
+
+    for idx, wth in enumerate(wth_vec):
+
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
+        c_op_list = []
+        rate = gamma1 * (1 + n_th)
+        c_op_list.append(np.sqrt(rate) * sm)
+        rate = gamma1 * n_th
+        c_op_list.append(np.sqrt(rate) * sm.dag())
+        rho_ss = steadystate(H, c_op_list, method='iterative-bicgstab')
+        p_ss[idx] = expect(sm.dag() * sm, rho_ss)
+
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-5, True)
 
@@ -156,28 +180,27 @@ def test_qubit_gmres():
 def test_qubit_lgmres():
     "Steady state: Thermal qubit - iterative-lgmres solver"
     # thermal steadystate of a qubit: compare numerics with analytical formula
-    sx = sigmax()
     sz = sigmaz()
     sm = destroy(2)
 
-    H = 0.5 * 2 * pi * sz
+    H = 0.5 * 2 * np.pi * sz
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * sm)
+        c_op_list.append(np.sqrt(rate) * sm)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * sm.dag())
+        c_op_list.append(np.sqrt(rate) * sm.dag())
         rho_ss = steadystate(H, c_op_list, method='iterative-lgmres')
         p_ss[idx] = expect(sm.dag() * sm, rho_ss)
 
-    p_ss_analytic = exp(-1.0 / wth_vec) / (1 + exp(-1.0 / wth_vec))
+    p_ss_analytic = np.exp(-1.0 / wth_vec) / (1 + np.exp(-1.0 / wth_vec))
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-5, True)
 
@@ -187,24 +210,24 @@ def test_ho_direct():
     # thermal steadystate of an oscillator: compare numerics with analytical
     # formula
     a = destroy(40)
-    H = 0.5 * 2 * pi * a.dag() * a
+    H = 0.5 * 2 * np.pi * a.dag() * a
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * a.dag())
+        c_op_list.append(np.sqrt(rate) * a.dag())
         rho_ss = steadystate(H, c_op_list, method='direct')
-        p_ss[idx] = real(expect(a.dag() * a, rho_ss))
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
 
-    p_ss_analytic = 1.0 / (exp(1.0 / wth_vec) - 1)
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-3, True)
 
@@ -214,24 +237,24 @@ def test_ho_eigen():
     # thermal steadystate of an oscillator: compare numerics with analytical
     # formula
     a = destroy(40)
-    H = 0.5 * 2 * pi * a.dag() * a
+    H = 0.5 * 2 * np.pi * a.dag() * a
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * a.dag())
+        c_op_list.append(np.sqrt(rate) * a.dag())
         rho_ss = steadystate(H, c_op_list, method='eigen')
-        p_ss[idx] = real(expect(a.dag() * a, rho_ss))
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
 
-    p_ss_analytic = 1.0 / (exp(1.0 / wth_vec) - 1)
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-3, True)
 
@@ -241,24 +264,24 @@ def test_ho_power():
     # thermal steadystate of an oscillator: compare numerics with analytical
     # formula
     a = destroy(40)
-    H = 0.5 * 2 * pi * a.dag() * a
+    H = 0.5 * 2 * np.pi * a.dag() * a
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * a.dag())
+        c_op_list.append(np.sqrt(rate) * a.dag())
         rho_ss = steadystate(H, c_op_list, method='power')
-        p_ss[idx] = real(expect(a.dag() * a, rho_ss))
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
 
-    p_ss_analytic = 1.0 / (exp(1.0 / wth_vec) - 1)
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-3, True)
 
@@ -268,24 +291,51 @@ def test_ho_gmres():
     # thermal steadystate of an oscillator: compare numerics with analytical
     # formula
     a = destroy(40)
-    H = 0.5 * 2 * pi * a.dag() * a
+    H = 0.5 * 2 * np.pi * a.dag() * a
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * a.dag())
+        c_op_list.append(np.sqrt(rate) * a.dag())
         rho_ss = steadystate(H, c_op_list, method='iterative-gmres')
-        p_ss[idx] = real(expect(a.dag() * a, rho_ss))
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
 
-    p_ss_analytic = 1.0 / (exp(1.0 / wth_vec) - 1)
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
+    delta = sum(abs(p_ss_analytic - p_ss))
+    assert_equal(delta < 1e-3, True)
+
+
+def test_ho_bicgstab():
+    "Steady state: Thermal HO - iterative-bicgstab solver"
+    # thermal steadystate of an oscillator: compare numerics with analytical
+    # formula
+    a = destroy(40)
+    H = 0.5 * 2 * np.pi * a.dag() * a
+    gamma1 = 0.05
+
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
+
+    for idx, wth in enumerate(wth_vec):
+
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
+        c_op_list = []
+        rate = gamma1 * (1 + n_th)
+        c_op_list.append(np.sqrt(rate) * a)
+        rate = gamma1 * n_th
+        c_op_list.append(np.sqrt(rate) * a.dag())
+        rho_ss = steadystate(H, c_op_list, method='iterative-bicgstab')
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
+
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-3, True)
 
@@ -295,24 +345,24 @@ def test_ho_lgmres():
     # thermal steadystate of an oscillator: compare numerics with analytical
     # formula
     a = destroy(40)
-    H = 0.5 * 2 * pi * a.dag() * a
+    H = 0.5 * 2 * np.pi * a.dag() * a
     gamma1 = 0.05
 
-    wth_vec = linspace(0.1, 3, 20)
-    p_ss = zeros(shape(wth_vec))
+    wth_vec = np.linspace(0.1, 3, 20)
+    p_ss = np.zeros(np.shape(wth_vec))
 
     for idx, wth in enumerate(wth_vec):
 
-        n_th = 1.0 / (exp(1.0 / wth) - 1)  # bath temperature
+        n_th = 1.0 / (np.exp(1.0 / wth) - 1)  # bath temperature
         c_op_list = []
         rate = gamma1 * (1 + n_th)
-        c_op_list.append(sqrt(rate) * a)
+        c_op_list.append(np.sqrt(rate) * a)
         rate = gamma1 * n_th
-        c_op_list.append(sqrt(rate) * a.dag())
+        c_op_list.append(np.sqrt(rate) * a.dag())
         rho_ss = steadystate(H, c_op_list, method='iterative-lgmres')
-        p_ss[idx] = real(expect(a.dag() * a, rho_ss))
+        p_ss[idx] = np.real(expect(a.dag() * a, rho_ss))
 
-    p_ss_analytic = 1.0 / (exp(1.0 / wth_vec) - 1)
+    p_ss_analytic = 1.0 / (np.exp(1.0 / wth_vec) - 1)
     delta = sum(abs(p_ss_analytic - p_ss))
     assert_equal(delta < 1e-3, True)
 
@@ -321,12 +371,12 @@ def test_driven_cavity_direct():
     "Steady state: Driven cavity - direct solver"
 
     N = 30
-    Omega = 0.01 * 2 * pi
+    Omega = 0.01 * 2 * np.pi
     Gamma = 0.05
 
     a = destroy(N)
     H = Omega * (a.dag() + a)
-    c_ops = [sqrt(Gamma) * a]
+    c_ops = [np.sqrt(Gamma) * a]
 
     rho_ss = steadystate(H, c_ops, method='direct')
     rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
@@ -338,12 +388,12 @@ def test_driven_cavity_eigen():
     "Steady state: Driven cavity - eigen solver"
 
     N = 30
-    Omega = 0.01 * 2 * pi
+    Omega = 0.01 * 2 * np.pi
     Gamma = 0.05
 
     a = destroy(N)
     H = Omega * (a.dag() + a)
-    c_ops = [sqrt(Gamma) * a]
+    c_ops = [np.sqrt(Gamma) * a]
 
     rho_ss = steadystate(H, c_ops, method='eigen')
     rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
@@ -355,12 +405,12 @@ def test_driven_cavity_power():
     "Steady state: Driven cavity - power solver"
 
     N = 30
-    Omega = 0.01 * 2 * pi
+    Omega = 0.01 * 2 * np.pi
     Gamma = 0.05
 
     a = destroy(N)
     H = Omega * (a.dag() + a)
-    c_ops = [sqrt(Gamma) * a]
+    c_ops = [np.sqrt(Gamma) * a]
 
     rho_ss = steadystate(H, c_ops, method='power')
     rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
@@ -372,12 +422,12 @@ def test_driven_cavity_gmres():
     "Steady state: Driven cavity - iterative-gmres solver"
 
     N = 30
-    Omega = 0.01 * 2 * pi
+    Omega = 0.01 * 2 * np.pi
     Gamma = 0.05
 
     a = destroy(N)
     H = Omega * (a.dag() + a)
-    c_ops = [sqrt(Gamma) * a]
+    c_ops = [np.sqrt(Gamma) * a]
 
     rho_ss = steadystate(H, c_ops, method='iterative-gmres')
     rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
@@ -385,16 +435,33 @@ def test_driven_cavity_gmres():
     assert_((rho_ss - rho_ss_analytic).norm() < 1e-4)
 
 
+def test_driven_cavity_bicgstab():
+    "Steady state: Driven cavity - iterative-bicgstab solver"
+
+    N = 30
+    Omega = 0.01 * 2 * np.pi
+    Gamma = 0.05
+
+    a = destroy(N)
+    H = Omega * (a.dag() + a)
+    c_ops = [np.sqrt(Gamma) * a]
+
+    rho_ss = steadystate(H, c_ops, method='iterative-bicgstab')
+    rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
+
+    assert_((rho_ss - rho_ss_analytic).norm() < 1e-4)
+
+
 def test_driven_cavity_lgmres():
     "Steady state: Driven cavity - iterative-lgmres solver"
 
     N = 30
-    Omega = 0.01 * 2 * pi
+    Omega = 0.01 * 2 * np.pi
     Gamma = 0.05
 
     a = destroy(N)
     H = Omega * (a.dag() + a)
-    c_ops = [sqrt(Gamma) * a]
+    c_ops = [np.sqrt(Gamma) * a]
 
     rho_ss = steadystate(H, c_ops, method='iterative-lgmres')
     rho_ss_analytic = coherent_dm(N, -1.0j * (Omega)/(Gamma/2))
